@@ -563,9 +563,9 @@ sc_signed::operator=(double v)
   }
   else
     sgn = SC_POS;
-  register int i = 0;
+  int i = 0;
   while (floor(v) && (i < ndigits)) {
-#ifndef WIN32
+#ifndef _WIN32
     digit[i++] = ((sc_digit)floor(remainder(v, DIGIT_RADIX))) & DIGIT_MASK;
 #else
     digit[i++] = ((sc_digit)floor(fmod(v, DIGIT_RADIX))) & DIGIT_MASK;
